package main

import (
	"context"
	"crypto/md5"
	"fmt"
	"log"
	"os"
	"path"

	"code.rocketnine.space/tslocum/desktop"
	"code.rocketnine.space/tslocum/gmenu/pkg/gmenu"
	"github.com/diamondburned/gotk4/pkg/gdkpixbuf/v2"
	"github.com/diamondburned/gotk4/pkg/gio/v2"
	"github.com/diamondburned/gotk4/pkg/gtk/v4"
)

const cachedIconCompressionLevel = 3

var (
	iconTheme    *gtk.IconTheme
	iconCacheDir string
)

func loadPNG(data []byte) (*gdkpixbuf.Pixbuf, error) {
	l, err := gdkpixbuf.NewPixbufLoaderWithType("png")
	if err != nil {
		return nil, fmt.Errorf("failed to initialize png loader: %s", err)
	}
	defer l.Close()

	if err = l.Write(data); err != nil {
		return nil, fmt.Errorf("failed to load png data: %s", err)
	}

	return l.Pixbuf(), nil
}

func loadIcon(icon string) (*gdkpixbuf.Pixbuf, error) {
	cachedIcon := path.Join(iconCacheDir, fmt.Sprintf("%x.png", md5.Sum([]byte(icon))))

	var (
		pbuf *gdkpixbuf.Pixbuf
		img  *gtk.Image
		err  error
	)
	if _, statErr := os.Stat(cachedIcon); !os.IsNotExist(statErr) {
		var bytes []byte
		bytes, err = os.ReadFile(cachedIcon)
		if err != nil {
			return nil, err
		}
		pbuf, err = loadPNG(bytes)
	}
	if pbuf == nil || err != nil {
		if path.IsAbs(icon) {
			var bytes []byte
			bytes, err = os.ReadFile(icon)
			if err != nil {
				return nil, err
			}
			pbuf, err = loadPNG(bytes)
		} else {
			iconPaintable := iconTheme.LookupIcon(icon, nil, iconSize, 1, gtk.TextDirLTR, gtk.IconLookupPreload)
			var bytes []byte
			bytes, err = os.ReadFile(iconPaintable.File().Path())
			if err != nil {
				return nil, err
			}
			pbuf, err = loadPNG(bytes)
		}
	}
	if err != nil {
		return nil, err
	}

	if pbuf != nil && img == nil {
		img = gtk.NewImageFromPixbuf(pbuf)
	}

	if pbuf.Width() != iconSize || pbuf.Height() != iconSize {
		pbuf = pbuf.ScaleSimple(iconSize, iconSize, gdkpixbuf.InterpBilinear)
	}

	go cacheIcon(cachedIcon, pbuf)

	return pbuf, nil
}

func cacheIcon(cachedIcon string, pbuf *gdkpixbuf.Pixbuf) {
	saveFile := gio.NewFileForPath(cachedIcon)
	saveFileStream, err := saveFile.Replace(context.Background(), "", false, gio.FileCreatePrivate)
	if err != nil {
		log.Printf("Failed to cache entry icon %s: %s", cachedIcon, err)
		return
	}
	pbuf.SaveToStreamv(context.Background(), saveFileStream, "png", nil, nil)
	saveFileStream.Close(context.Background())
}

func fallbackIcon(entry *gmenu.ListEntry) string {
	if entry.Entry == nil {
		return "utilities-terminal"
	} else if entry.Type == desktop.Application {
		return "application-x-executable"
	}

	return "text-html"
}

func loadIconImage(img *gtk.Image, entry *gmenu.ListEntry) {
	var (
		pbuf *gdkpixbuf.Pixbuf
		err  error
	)
	if entry.Entry != nil && entry.Icon != "" {
		pbuf, err = loadIcon(entry.Icon)
	}
	if pbuf == nil || err != nil {
		pbuf, err = loadIcon(fallbackIcon(entry))
	}
	if err != nil {
		log.Printf("failed to load entry icon %s: %s", entry.Icon, err)
	}

	img.SetFromPixbuf(pbuf)
}
